<?php

require_once("classes/translator.class.php");
tl::register("faxserver", dirname(__FILE__)."/lang");

require_once("app-classes/innovaphone.fax.database.class.php");
require_once("app-classes/innovaphone.fax.config.class.php");
require_once("app-classes/innovaphone.fax.shm.class.php");
require_once("include_main.php");
require_once("postgresql.php");


class FaxTl {
    static $initialized = false;
    static $translations = array();
    
    static function init() {
        if(self::$initialized) {
            return;
        }
        self::$translations = array(
            'incoming'                      => tl::tl('incoming'),
            'outgoing'                      => tl::tl('outgoing'),
            'created'                       => tl::tl('created'),
            'queued'                        => tl::tl('queued'),
            'closed'                        => tl::tl('closed'),
            'waiting'                       => tl::tl('waiting'),
            'calling'                       => tl::tl('calling'),
            'alerting'                      => tl::tl('alerting'),
            'connecting'                    => tl::tl('establishing of the fax connection'),
            'connected'                     => tl::tl('connected'),
            'transmitting'                  => tl::tl('transmitting'),
            'receiving'                     => tl::tl('receiving'),
            'completed'                     => tl::tl('completed'),
            'aborted'                       => tl::tl('aborted'),
            'error'                         => tl::tl('error'),

            'canceled'                      => tl::tl('canceled'),
            'by system'                     => tl::tl('application error'),
            'by user'                       => tl::tl('user error'),
            'response timeout'              => tl::tl('response timeout'),
            'busy'                          => tl::tl('busy'),
            'no channel available'          => tl::tl('no channel available'),
            'no FAX device'                 => tl::tl('no fax device'),
            'no fax device'                 => tl::tl('no fax device'),
            'call disconnected'             => tl::tl('call disconnected'),
            'call aborted'                  => tl::tl('call aborted'),
            'script error'                  => tl::tl('script error'),
            'no-response'                   => tl::tl('no response'),
            'timeout'                       => tl::tl('timeout during the connection'),
            'bad-connection'                => tl::tl('bad connection'),
            'by-remote'                     => tl::tl('disconnected by the remote'),
            'file-open'                     => tl::tl('file open error'),
            'file-read'                     => tl::tl('file read error'),
            'file-write'                    => tl::tl('file write error'),
            'file-type'                     => tl::tl('wrong file content'),
            'remote'                        => tl::tl('incompatible remote'),
            'protocol'                      => tl::tl('protocol error'),

            'NoFaxRecipients'               => tl::tl('no fax recipients'),
            'NoAttachment'                  => tl::tl('no mail attachment'),
            'ReadAttachment'                => tl::tl('attachment read error'),
            'Email address not configured'  => tl::tl('email address not configured'),
            'User not found'                => tl::tl('user not found'),
            'No user license'               => tl::tl('no user license'),
            'License check failed.'         => tl::tl('license check failed'),
            'retry timeout'                 => tl::tl('retry timeout'),
            'not created in PBX'            => tl::tl('not created in PBX'),
            'not found in PBX'              => tl::tl('not found in PBX'),
            'unexpected call end'           => tl::tl('unexpected call end'),
            'wrong fax call state'          => tl::tl('wrong fax call state'),

            'command'                       => tl::tl('command'),
            'failed'                        => tl::tl('failed'),
            'exit code'                     => tl::tl('exit code'),
            'release code'                  => tl::tl('release code'),

            'successful'                    => tl::tl('successful'),
            'file in use or system busy'    => tl::tl('file in use or system busy'),
            'file not found'                => tl::tl('file not found'),
            'file system not mounted'       => tl::tl('file system not mounted'),
            'operation not allowed'         => tl::tl('operation not allowed'),
            'authentication required'       => tl::tl('authentication required'),
            'file system out of space'      => tl::tl('file system out of space'),
            'file already exists'           => tl::tl('file already exists'),
            'invalid arguments'             => tl::tl('invalid arguments'),
            'target is a directory'         => tl::tl('target is a directory'),
            'operation not supported'       => tl::tl('operation not supported'),
            'unspecified error'             => tl::tl('unspecified error'),
            );
        
        self::$initialized = true;
    }

    static function tl($text) {
        if(self::$initialized == false) {
            self::init();
        }
        $text = strtr($text, self::$translations);
        return $text;   
    }
}

class FaxConfigConsts
{
    const pageConfig = 1;
    const tabPgsql = "pgsql", tabServer = "server", tabAuthorizedHosts = "authHosts";
    const modeInRePgPw = "inrepgpw",
          modeDeleteServer = "deleteserver", 
          modeServer1 = "configserver1",
          modeServer2 = "configserver2",
          modeServer3 = "configserver3",
          modeServer4 = "configserver4",
          modeServerStrings1 = "configserverstrings1",
          modeServerStrings2 = "configserverstrings2",
          modeServerCovers = "servercovers",
          modeServerCoversAdd = "servercoversadd",
          modeServerCoversDelete = "servercoversdelete",
          modeAuthorizedHostsDefault = "authHostsDefault";
}

class FaxInputPagePhoneField extends InputPagePhoneField {
    public function __construct($name, $prompt = "", $default = "", InputPageAction $button = null) {
        parent::__construct($name, $prompt, $default, $button);
        $this->pattern = "(".$this->pattern.")|(^\+$)";
    }
}

class InputPageTextInfo extends genericInputPageItem {
    
    // hmtl tag
    var $type = "div";
    var $info = "";
    
    /**
     * a single text area in the form.
     * used with class InputPage, not passed between forms
     * @param string $name
     * @param string $value
     * @param string $info
     * @param string $type 
     */
    public function InputPageTextInfo($name, $value, $info, $type = "div") {
        $this->genericInputPageItem($name, $value);
        $this->type = $type;
        $this->info = $info;
        $this->hint = "";
        $this->sortType = genericInputPageItem::sortIgnore;
    }
    
    // render the field in $columns columns
    public function render($columns) {
        if ($this->hidden) return "";
        $tdattribs = html::formatAttributes($this->tdattributes);
        $asyncTdEvents = (is_object($this->rootPage) ? $this->rootPage->formatAsyncEvents($this->tdEvents) : "");
        $asyncEvents = (is_object($this->rootPage) ? $this->rootPage->formatAsyncEvents($this->events) : "");
        $class = "";
        if(!isset($this->tdattributes["class"])) {
            $class = "class='hint'";
        }
        if($columns <= 1) {
            return "     <td $class title='" . html::hq($this->getHint()) . "' {$tdattribs} " . $asyncTdEvents . ">"
            . "<" . $this->type . " "
            . html::formatAttributes($this->attributes)
            . $asyncEvents
            . " name='" . $this->name . "'>"
            . $this->value 
            . "</" . $this->type . ">" 
            . "</td>";
        }
        $columns--;
        return "     <td $class title='" . html::hq($this->getHint()) . "' {$tdattribs} " . $asyncTdEvents . ">"
            . "<" . $this->type . " "
            . html::formatAttributes($this->attributes)
            . $asyncEvents
            . " name='" . $this->name . "'>"
            . $this->value 
            . "</" . $this->type . ">" 
            . "</td>"
            . "     <td colspan='$columns' $class title='" . html::hq($this->getHint()) . "' {$tdattribs} " . $asyncTdEvents . ">"
            . "<" . $this->type . " "
            . html::formatAttributes($this->attributes)
            . $asyncEvents
            . " name='" . $this->name . "_info'>"
            . $this->info
            . "</" . $this->type . ">" 
            . "</td>"; 
    }
    
    public function dumptext() { return false; }
    
    // text areas are always valid
    function valid() {
        return true;
    }
}

class InputPageLoginStringField extends InputPageStringField {
    public function InputPageLoginStringField($name, $prompt = "", $default = "", InputPageAction $button = null) {
        parent::__construct($name, $prompt, $default, $button);
    }
    
    public function valid() {
        if(!parent::valid()) return false;

        if(preg_match('/[^0-9A-Za-z.\-_]/', $this->value)) {
            $this->msg = tl::tlx('The character {$arg[1]} is not allowed.', mb_substr(preg_replace('/[0-9A-Za-z.\-_]/', "", $this->value), 0, 1, html::$encoding)) . 
                         ' ' . tl::tl('Only these lower case characters can be used:') . ' a-z 0-9 .-_';
            $this->value = strtolower($this->value);
            return false;
        }
        if(preg_match('/[^0-9a-z.\-_]/', $this->value)) {
            $this->msg = tl::tl('Upper case characters are not allowed.') . ' ' . tl::tl('The string has been changed.');
            $this->value = strtolower($this->value);
            return false;
        }
        return true;
    }
}

class ButtonShowCalls extends InputPageButtonOnlyAction {
    
    public function ButtonShowCalls($name, $value) {
        parent::__construct($name, $value);
    }

    function action(InputPage &$page, $phase) {
        $page->render();
    }
}

abstract class FaxInputPage extends MainInnovaphonePage 
{
    public function FaxInputPage($site, $name, $printMenu = true) {
        $this->enableTranslations = true;
        parent::MainInnovaphonePage($site, $name, $printMenu);
        $this->logoImg = "app-images/logo_faxserver.png";
        // add java script file
        $this->scriptfiles += array("javafax" => "java.js");
        //$this->links[count($this->links)] = array("rel" => "stylesheet", "type" => "text/css", "href" => "style_reporting.css");        
        
        // default pageno if not set (1)
        if (($this->pageNo = $this->page()) == "") $this->pageNo = 1;        
        $version = 0;
        $hotfix = 0;
        MainInputPage::getVersions(Config::webRoot, $version, $hotfix);
        global $faxSession;
        $app = $faxSession->mainSession->getApp("innovaphone-faxserver");
        if(((int)$app->hotfix + (int)$app->hotfixOffset) < (int)$hotfix) {
            die(tl::tlx('Install innovaphone-faxserver hotfix{$arg[1]} first!', $hotfix));
            exit;
        }
        $this->searchArea = "<br><br><b>{$app->versionStr}</b>";
    }
    
    function addMenu() {
        $this->addMainMenuItem(tl::tl("Config"), "config_faxserver.php", $this->name == "faxconfig");
        $this->addMainMenuItem(tl::tl("Diagnostics"), "diagnostics.php", $this->name == "innovaphonediagnostics");
    }    
    
    function getHelpLink() {
        return Config::wikiHelpFaxserver.$this->pages[$this->pageNo]->getHelpAnchor();
    }
    
    function tabFaxCalls(MyPage &$page) {
        $page->currentSection = InnovaphoneDiagnosticsConsts::tabFaxCalls;
        $dbCalls = $page->addField(new InputPageText("dbcalls", $page->getModeLink(InnovaphoneDiagnosticsConsts::modeFaxDbCalls, tl::tl("List Database Calls"))));
        $dbCalls->fullline = true;
        
        $shmCalls = $page->addField(new InputPageText("shmcalls", $page->getModeLink(InnovaphoneDiagnosticsConsts::modeFaxShmCalls, tl::tl("List Running Calls"))));
        $shmCalls->fullline = true;
        
        switch($this->currentMode) {
            case InnovaphoneDiagnosticsConsts::modeFaxDbCalls:
                $this->addModeFaxDbCalls($page);
                break;
            case InnovaphoneDiagnosticsConsts::modeFaxShmCalls:
                $this->addModeFaxShmCalls($page);
                break;
        }
    }
    
    public function addModeFaxShmCalls(MyPage &$page) {
        $this->metas += array("refresh" => "2; URL=".InputPage::relativeScriptName().
                    "?_page=".  InnovaphoneDiagnosticsConsts::pageDiagnostics.
                    "&tab=".  InnovaphoneDiagnosticsConsts::tabFaxCalls.
                    "&mode=". InnovaphoneDiagnosticsConsts::modeFaxShmCalls);
        
        $page->currentSection = $this->currentMode;
        $page->addSection($this->currentMode, 1, true, tl::tl("Calls"));
        $list = $page->addField(new MainInputPageListField("listcalls"));
        $emptyText = tl::tl("No calls");
        $empty = $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable",
            "<b>$emptyText</b>");
        $empty->style = $page->currentSection;
        $list->addColumn($server = new InputPageText($list->name."server", null), tl::tl("Server"));
        $server->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($time = new MainInputPageNoBoxDateField($list->name."time", null, 0, null, true), tl::tl("Time"));
        $list->addColumn($user = new InputPageText($list->name."user", null), tl::tl("User"));
        $user->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($number = new InputPageText($list->name."number", null), tl::tl("Number"));
        $number->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($remote = new InputPageText($list->name."remotenumber", null), tl::tl("Remote Number"));
        $remote->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($dir = new InputPageText($list->name."direction", null), tl::tl("Direction"));
        $dir->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($state = new InputPageText($list->name."state", null), tl::tl("State"));
        $state->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($page = new InputPageText($list->name."page", null), tl::tl("Page"));
        $page->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($duration = new InputPageText($list->name."duration", null), tl::tl("Duration"));
        $duration->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($error = new InputPageText($list->name."error", null), tl::tl("Error"));
        $error->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
    }
    
    public function addModeFaxDbCalls(MyPage &$page) {
        $page->currentSection = $this->currentMode;
        $page->addSection($this->currentMode, 2, true, tl::tl("Calls"));
        $page->addField($dateStart = new InputPageCalendarDateField("from", tl::tl("From")));
        $page->addField($dateEnd = new InputPageCalendarDateField("to", tl::tl("To")));
        $page->addSubSection($this->currentMode, "_btn");
        $page->addField($btn = new ButtonShowCalls("btnshow", tl::tl("Ok")));
        $page->addField(new InputPageHiddenIdField("job_action"));
        $page->addField(new InputPageHiddenIdField("job_server_id"));
        $page->addField(new InputPageHiddenIdField("job_guid"));
        $btn->fullline = true;
        $btn->attributes["style"] = "margin-bottom:20px";
        $list = $page->addField(new MainInputPageListField("listcalls"));
        $emptyText = tl::tl("No calls");
        $empty = $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable",
            "<b>$emptyText</b>");
        $empty->style = $page->currentSection;
        $list->addColumn($server = new InputPageText($list->name."server", null), tl::tl("Server"));
        $server->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($time = new MainInputPageNoBoxDateField($list->name."time", null, 0, null, true), tl::tl("Time"));
        $list->addColumn($user = new InputPageText($list->name."user", null), tl::tl("User"));
        $user->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($number = new InputPageText($list->name."number", null), tl::tl("Number"));
        $number->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($remote = new InputPageText($list->name."remotenumber", null), tl::tl("Remote Number"));
        $remote->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($dir = new InputPageText($list->name."direction", null), tl::tl("Direction"));
        $dir->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($subId = new InputPageText($list->name."subscriber", null), "Subscriber ID");
        $subId->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($progress = new InputPageText($list->name."progress", null), tl::tl("Progress"));
        $progress->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($page = new InputPageText($list->name."pages", null), tl::tl("Pages"));
        $page->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($duration = new InputPageText($list->name."duration", null), tl::tl("Duration"));
        $duration->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($error = new InputPageText($list->name."error", null), tl::tl("Error"));
        $error->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($cancel = new InputPageText($list->name."action", null), "");
    }
    
    function fillListFaxShmCalls() {
        $faxDb = get_fax_db();
        $list = $this->getField("listcalls");
        $list->value = array();
        $query = "SELECT shm_id FROM ".InnovaphoneFaxDatabase::tableServers." 
                    WHERE active=true";
        $result = $faxDb->query($query);
        $faxCalls = array();
        $query = "SELECT * FROM ".InnovaphoneFaxDatabase::tableFaxJobs." j
                    JOIN ".InnovaphoneFaxDatabase::tableServers." s ON (j.server_id=s.id) 
                    WHERE guid IN (''";
        while(($row = pg_fetch_object($result))) {
            $faxShm = new InnovaphoneFaxShm($row->shm_id);
            $faxShm->read();
            $calls = $faxShm->getCalls();
            $jobs = $faxShm->getJobs();
            foreach($jobs as $guid => /*@var $job InnovaphoneFaxShmJobData*/$job) {
                $confId = $job->getConfId();
                $faxCalls[$guid] = null;
                $query .= ",'$guid'";
                if($confId != null) {
                    foreach($calls as $confCall => /*@var $call InnovaphoneFaxShmCallData*/$call) {
                        if($confCall == $confId) {
                            $faxCalls[$guid] = $call;
                            break;
                        }
                    }
                }
            }
        }
        $query .= ")";
        if(!count($faxCalls)) return;
        $result = $faxDb->query($query);
        $rows = 0;
        while(($row = pg_fetch_object($result))) {
            /*@var $call InnovaphoneFaxShmCallData*/$call = $faxCalls[$row->guid];
            $ts = strtotime($row->timestamp);
            $duration = time() - $ts;
            $strDuration = sprintf("%02d:%02d:%02d", $duration/(60*60), ($duration/60)%60, $duration%60);
            $list->value[$rows][$list->name."server"] = $row->name;
            $list->value[$rows][$list->name."time"] = $ts;
            $list->value[$rows][$list->name."user"] = $row->user_name;
            $list->value[$rows][$list->name."number"] = $row->user_number;
            $list->value[$rows][$list->name."remotenumber"] = $row->remote_number;
            $list->value[$rows][$list->name."direction"] = FaxTl::tl($row->direction);
            $list->value[$rows][$list->name."duration"] = $strDuration;
            if($call == null) {
                $list->value[$rows][$list->name."state"] = FaxTl::tl($row->progress);
                $list->value[$rows][$list->name."page"] = "";
                $list->value[$rows][$list->name."error"] = FaxTl::tl($row->error_cause);
                if($row->error_description != "") {
                    $list->value[$rows][$list->name."error"] .= ": ". FaxTl::tl($row->error_description);
                }
            }
            else {
                $list->value[$rows][$list->name."state"] = FaxTl::tl($call->getFaxState());
                $list->value[$rows][$list->name."error"] = FaxTl::tl($call->getErrorCause());
                $desc = $call->getErrorDescription();
                if($desc != "") {
                    $list->value[$rows][$list->name."error"] .= ": ".FaxTl::tl($desc);
                }
                $list->value[$rows][$list->name."page"] = $call->getPage();
                $pages = $call->getPages();
                if($pages > 0) {
                    $list->value[$rows][$list->name."page"] .= "/".$pages;
                }
            }
            $rows++;
        }
    }
    
    function fillListFaxDbCalls() {
        $dateStart = $this->getField("from");
        $dateEnd = $this->getField("to");
        $faxDb = get_fax_db();
        $rows = 0;
        $list = $this->getField("listcalls");
        $list->value = array();
        
        $jobAction      = $this->getField("job_action")->value;
        $jobServerId    = $this->getField("job_server_id")->value;
        $jobGuid        = $this->getField("job_guid")->value;
        
        $this->getField("job_action")->value = "";
        $this->getField("job_server_id")->value = "";
        $this->getField("job_guid")->value = "";
        
        $actionResult = false;
        if($jobAction != "") {
            $actionResult = $this->doJobAction($faxDb, $jobAction, $jobServerId, $jobGuid);
        }
        
        if(!$dateStart->sent) {
            $list->fieldOnEmptyTable->value = "";
            return;
        }
        
        $time = strtotime("-1 month");
        $query = "SELECT * FROM ".InnovaphoneFaxDatabase::tableFaxJobs." AS j
                    LEFT OUTER JOIN ".InnovaphoneFaxDatabase::tableServers." AS s ON (j.server_id=s.id)
                    WHERE timestamp>='".date("Y-m-d", $dateStart->value)."' AND timestamp<'".date("Y-m-d", strtotime("+1 day", $dateEnd->value))."'";
        $result = $faxDb->query($query);
        while(($row = pg_fetch_object($result))) {
            $list->value[$rows][$list->name."server"] = $row->name;
            $list->value[$rows][$list->name."time"] = strtotime($row->timestamp);
            $list->value[$rows][$list->name."user"] = $row->user_name;
            $list->value[$rows][$list->name."number"] = $row->user_number;
            $list->value[$rows][$list->name."remotenumber"] = $row->remote_number;
            $list->value[$rows][$list->name."direction"] = FaxTl::tl($row->direction);
            $list->value[$rows][$list->name."duration"] = $row->duration;
            $list->value[$rows][$list->name."subscriber"] = $row->remote_subscriber_id;
            $list->value[$rows][$list->name."progress"] = FaxTl::tl($row->progress);
            $list->value[$rows][$list->name."pages"] = ($row->pages ? $row->pages : "");
            $list->value[$rows][$list->name."error"] = FaxTl::tl($row->error_cause);
            if($row->error_description != "") {
                $list->value[$rows][$list->name."error"] .= ": ".FaxTl::tl($row->error_description);
            }
            $list->value[$rows][$list->name."action"] = "";
            if($row->progress == "queued" && $row->direction == "outgoing" && $row->result != "error" && $row->result != "success") {
                $list->value[$rows][$list->name."action"] .= "<a href=\"javascript:submitJobAction('cancel', '".
                    $row->server_id . "', '" . $row->guid . "');\">" . tl::tl("Cancel") ."</a>";
            }
            if($actionResult !== false && $jobServerId == $row->server_id && $jobGuid == $row->guid) {
                $list->value[$rows][$list->name."action"] .= ($actionResult === true ? "" : (" ".tl::tl("failed").": ".$actionResult));
            }
            $rows++;
        }
    }
    
    function fillListFax() {
        switch($this->currentMode) {
            case InnovaphoneDiagnosticsConsts::modeFaxDbCalls:
                $this->fillListFaxDbCalls();
                break;
            case InnovaphoneDiagnosticsConsts::modeFaxShmCalls:
                $this->fillListFaxShmCalls();
                break;
        }
    }
    
    function doJobAction(InnovaphoneFaxDatabase &$faxDb, $jobAction, $jobServerId, $jobGuid) {
        $ret = "wrong action";
        if($jobAction == 'cancel') {
            $shmId = $faxDb->getShmId($jobServerId);
            $faxShm = new InnovaphoneFaxShm($shmId);
            if($faxShm->lockJob($jobGuid, null, null)) {
                $faxDb->setServerId($jobServerId);
                $faxJob = $faxDb->getFaxJob($jobGuid);
                if($faxJob) {
                    if($faxJob->getProgress() == 'queued' &&
                        $faxJob->getDirection() == "outgoing" &&
                        $faxJob->getResult() != "error" &&
                        $faxJob->getResult() != "success")
                    {
                        $faxJob->setResult('error');
                        $faxJob->setErrorCause('canceled');
                        $faxJob->setErrorDescription(null);
                        $faxJob->setDelayed(null);
                        $faxJob = $faxDb->updateFaxJob($faxJob);
                        $ret = true;
                        if(!$faxJob) {
                            $ret = tl::tl("job update failure");
                        }
                    }
                    else {
                        $ret = tl::tl("job cannot be canceled");
                    }
                }
                else {
                    $ret = tl::tl("job not found");
                }
                $faxShm->unlockJob($jobGuid);
            }
            else {
                $ret = tl::tl("job in process");
            }
        }
        return $ret;
    }
}

class FaxXmlCfg {
    var $error = "";
    var $mtime = 0;
    
    var $trace = false;
    
    public function FaxXmlCfg() {
        $this->readConfig();
    }
    
    public function readConfig($file = Config::faxConfig, $import = false) {
        if(!$import) {
            
        }
        $doc = new DOMDocument();
        $doc->encoding = "utf-8";
        $doc->preserveWhiteSpace = false;
        @$doc->load($file);
        if($doc !== false) {
            if(!$import && file_exists($file)) {
                $this->mtime = filemtime($file);
            }
            $config = $doc->getElementsByTagName ("config");
            if($config->length > 0) {
                $config = $config->item(0);
                foreach($config->childNodes as $node) {
                    switch($node->nodeName) {
                        case "trace":
                            $this->trace = ($node->textContent == "1" ? true : false); break;
                            break;
                        default:
                            break;
                    }
                }
            }
            else {
                $this->error = tl::tlx('Config file {$arg[1]} does not contain a <config> tag.', $file);
                return false;
            }
        }
        else {
            $this->error = tl::tlx('Couldn\'t parse config file {$arg[1]}.', $file);
            return false;
        }
        return true;
    }
    
    public function writeConfig() {
        $doc = new DOMDocument("1.0", "utf-8");
        $doc->formatOutput = true;
        $config = $doc->appendChild($doc->createElement("config"));
        
        // trace flag
        $config->appendChild($doc->createElement("trace", ($this->trace ? "1" : "0")));
        
        // save doc
        if($doc->save(Config::faxConfig) === false) {
            $this->error = tl::tlx('Couldn\'t rewrite fax server config \'{$arg[1]}\'.', Config::faxConfig);
            return false;
        }
    }
    
    public function write() {
        if($this->writeConfig() === false) {
            return false;
        }
        
        global $faxSession;
        $faxSession->updateFaxCfg($this);
        return true;
    }
}

class FaxSession {
    /**
     * FaxConfig
     *
     * @var FaxXmlConfig 
     *
     */    
    var $faxCfg = null;
    
    /**
     *
     * @var FaxServer
     */
    var $faxServer = null;
    
    var $mainSession;
    
    public function FaxSession() {
        global $mainSession;
        $this->mainSession = $mainSession;
        $this->faxServer = new FaxServer();
        if($this->mainSession->restore($this->faxServer, "faxserver", false) === false) {
            $this->faxCfg = new FaxXmlCfg();
            $this->mainSession->save($this->faxCfg, "faxcfg");
            $this->mainSession->save($this->faxServer, "faxserver");
        }
        $this->mainSession->restore($this->faxCfg, "faxcfg");
        $this->mainSession->restore($this->faxServer, "faxserver");
        $this->checkFaxCfg();
    }
    
    function checkFaxCfg() {
        if(file_exists(Config::faxConfig)) {
            $mtime = filemtime(Config::faxConfig);
            if($this->faxCfg == null || $mtime != $this->faxCfg->mtime) {
                $this->updateFaxCfg(new FaxXmlCfg());
            }
        }
    }
    
    function updateFaxCfg($cfg) {
        $this->faxCfg = $cfg;
        $this->mainSession->save($this->faxCfg, "faxcfg");
    }
    
    function updateServer($faxServer) {
        $this->faxServer = $faxServer;
        $this->mainSession->save($this->faxServer, "faxserver");
    }
}

class FaxMethods {
    
}

/* @var $faxDb InnovaphoneFaxDatabase */ $faxDb = null;
/**
 *
 * @global InnovaphoneFaxDatabase $faxDb
 * @return \InnovaphoneFaxDatabase 
 */
function get_fax_db() {
    global $faxDb;
    if ($faxDb == null) {
        $faxDb = new InnovaphoneFaxDatabase();
        if (!$faxDb->isConnected()) {
            home_die(tl::tl("No postgresql db available."), "config_faxserver.php");
            exit;
        }
    }
    return $faxDb;
}

/*function send_alarm_to_pbx($text, $msg, $errorCode) {
    global $faxSession;
    $uri = "http://{$faxSession->exchangeCfg->pbx->ip}/LOG0/FAULT/post-msg";
    $link = utf8_encode($uri);
    
    $datetime = date("U");
    // define fault code in log_if.h!
    $vars = "fault_xml_post <fault type=\"alarm\" severity=\"1\" active=\"false\" code=\"0x0018000".dechex($errorCode)."\" date=\"".$datetime."\" host=\"".$_SERVER['HTTP_HOST']."\" xsl=\"fault_reporting.xsl\"><text>";
    $vars .= xmlentities("$text: $msg");
    $vars .= "</text><source>LINUX-Exchange</source><details>";
    //$vars .= $msg;
    $vars .= "</details></fault>";
    
    $c = curl_init();
    curl_setopt($c, CURLOPT_HEADER, 1);
    curl_setopt($c, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
    curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($c, CURLOPT_URL, $link);
    curl_setopt($c, CURLOPT_POST, 1);
    curl_setopt($c, CURLOPT_POSTFIELDS, $vars);                                                         
    
    $response = curl_exec($c);
    curl_close($c);
}*/

/* @var $faxSession FaxSession */$faxSession = new FaxSession();
if($faxSession->mainSession->error) {
    $faxSession->mainSession->destroy();
    home_die($faxSession->mainSession->error, "config_faxserver.php");
}

?>